@@ -65,15 +65,13 @@ class CompilationError < RuntimeError; end
 # the current namespace.
 
 module Inline
-  VERSION = '3.11.2'
+  VERSION = "3.12.2"
 
   WINDOZE  = /mswin|mingw/ =~ RUBY_PLATFORM
   RUBINIUS = defined? RUBY_ENGINE
   DEV_NULL = (WINDOZE ? 'nul'      : '/dev/null')
-  GEM      = (WINDOZE ? 'gem.bat'  : 'gem')
-  RAKE     = if WINDOZE then
-               'rake.bat'
-             elsif RUBINIUS then
+  GEM      = 'gem'
+  RAKE     = if RUBINIUS then
                File.join(Gem.bindir, 'rake')
              else
                "#{Gem.ruby} -S rake"
@@ -293,7 +291,8 @@ module Inline
                 0
               end
 
-      file, line = caller[1].split(/:/)
+      file, line = $1, $2 if caller[1] =~ /(.*?):(\d+)/
+
       result = "# line #{line.to_i + delta} \"#{file}\"\n" + result unless
         $DEBUG and not $TESTING
 
@@ -318,6 +317,10 @@ module Inline
 
       ext << @inc
       ext << nil
+      unless @pre.empty? then
+        ext << @pre.join("\n\n")
+        ext << nil
+      end
       ext << @src.join("\n\n")
       ext << nil
       ext << nil
@@ -369,6 +372,7 @@ module Inline
       unless defined? @module_name then
         module_name = @mod.name.gsub('::','__')
         md5 = Digest::MD5.new
+        @pre.each { |m| md5 << m.to_s }
         @sig.keys.sort_by { |x| x.to_s }.each { |m| md5 << m.to_s }
         @module_name = "Inline_#{module_name}_#{md5}"
       end
@@ -384,7 +388,7 @@ module Inline
 
     attr_reader :rb_file, :mod
     attr_writer :mod
-    attr_accessor :src, :sig, :flags, :libs, :init_extra
+    attr_accessor :src, :pre, :sig, :flags, :libs, :init_extra
 
     ##
     # Sets the name of the C struct for generating accessors.  Used with
@@ -405,6 +409,7 @@ module Inline
       @rb_file = File.expand_path real_caller
 
       @mod = mod
+      @pre = []
       @src = []
       @inc = []
       @sig = {}
@@ -552,10 +557,10 @@ VALUE #{method}_equals(VALUE value) {
 
         if recompile then
 
-          hdrdir = %w(srcdir archdir rubyhdrdir).map { |name|
+          hdrdir = %w(srcdir includedir archdir rubyhdrdir).map { |name|
             RbConfig::CONFIG[name]
           }.find { |dir|
-            dir and File.exist? File.join(dir, "/ruby.h")
+            dir and File.exist? File.join(dir, "ruby.h")
           } or abort "ERROR: Can't find header dir for ruby. Exiting..."
 
           flags = @flags.join(' ')
@@ -567,32 +572,44 @@ VALUE #{method}_equals(VALUE value) {
                             nil
                           end
 
+          windoze = WINDOZE and RUBY_PLATFORM =~ /mswin/
+          sane = ! windoze
           cmd = [ RbConfig::CONFIG['LDSHARED'],
                   flags,
-                  RbConfig::CONFIG['DLDFLAGS'],
-                  RbConfig::CONFIG['CCDLFLAGS'],
+                  (RbConfig::CONFIG['DLDFLAGS']         if sane),
+                  (RbConfig::CONFIG['CCDLFLAGS']        if sane),
                   RbConfig::CONFIG['CFLAGS'],
-                  RbConfig::CONFIG['LDFLAGS'],
+                  (RbConfig::CONFIG['LDFLAGS']          if sane),
                   '-I', hdrdir,
                   config_hdrdir,
                   '-I', RbConfig::CONFIG['includedir'],
-                  "-L#{RbConfig::CONFIG['libdir']}",
-                  '-o', so_name.inspect,
+                  ("-L#{RbConfig::CONFIG['libdir']}"    if sane),
+                  (['-o', so_name.inspect]              if sane),
                   File.expand_path(src_name).inspect,
                   libs,
-                  crap_for_windoze ].join(' ')
+                  crap_for_windoze,
+                  (RbConfig::CONFIG['LDFLAGS']          if windoze),
+                  (RbConfig::CONFIG['CCDLFLAGS']        if windoze),
+                ].compact.join(' ')
 
          # strip off some makefile macros for mingw 1.9
          cmd = cmd.gsub(/\$\(.*\)/, '') if RUBY_PLATFORM =~ /mingw/
-        
+
           # TODO: remove after osx 10.5.2
           cmd += ' -flat_namespace -undefined suppress' if
             RUBY_PLATFORM =~ /darwin9\.[01]/
           cmd += " 2> #{DEV_NULL}" if $TESTING and not $DEBUG
 
           warn "Building #{so_name} with '#{cmd}'" if $DEBUG
-          result = `#{cmd}`
+
+          result = if WINDOZE
+                     Dir.chdir(Inline.directory) { `#{cmd}` }
+                   else
+                     `#{cmd}`
+                   end
+
           warn "Output:\n#{result}" if $DEBUG
+
           if $? != 0 then
             bad_src_name = src_name + ".bad"
             File.rename src_name, bad_src_name
@@ -630,10 +647,10 @@ VALUE #{method}_equals(VALUE value) {
       # gawd windoze land sucks
       case RUBY_PLATFORM
       when /mswin32/ then
-        " -link /LIBPATH:\"#{RbConfig::CONFIG['libdir']}\" /DEFAULTLIB:\"#{RbConfig::CONFIG['LIBRUBY']}\" /INCREMENTAL:no /EXPORT:Init_#{module_name}"
+        " -link /OUT:\"#{self.so_name}\" /LIBPATH:\"#{RbConfig::CONFIG['libdir']}\" /DEFAULTLIB:\"#{RbConfig::CONFIG['LIBRUBY']}\" /INCREMENTAL:no /EXPORT:Init_#{module_name}"
       when /mingw32/ then
         c = RbConfig::CONFIG
-        " -Wl,--enable-auto-import -L#{c['libdir']} -l#{c['RUBY_SO_NAME']}"
+        " -Wl,--enable-auto-import -L#{c['libdir']} -l#{c['RUBY_SO_NAME']} -o #{so_name.inspect}"
       when /i386-cygwin/ then
         ' -L/usr/local/lib -lruby.dll'
       else
@@ -650,6 +667,13 @@ VALUE #{method}_equals(VALUE value) {
       @flags.push(*flags)
     end
 
+    ##
+    # Registers a static id_name for the symbol :name.
+
+    def add_id name
+      self.add_static "id_#{name}", "rb_intern(\"#{name}\")"
+    end
+
     ##
     # Adds linker flags to the link command line.  No preprocessing is
     # done, so you must have all your dashes and everything.
@@ -699,12 +723,12 @@ VALUE #{method}_equals(VALUE value) {
     end
 
     ##
-    # Maps a ruby constant to C (with the same name)
+    # Maps RubyConstants to cRubyConstants.
 
     def map_ruby_const(*names)
       names.each do |name|
-        self.prefix "static VALUE #{name};"
-        self.add_to_init "    #{name} = rb_const_get(c, rb_intern(#{name.to_s.inspect}));"
+        self.prefix "static VALUE c#{name};"
+        self.add_to_init "    c#{name} = rb_const_get(c, rb_intern(#{name.to_s.inspect}));"
       end
     end
 
@@ -745,7 +769,7 @@ VALUE #{method}_equals(VALUE value) {
     # Adds any amount of text/code to the source
 
     def prefix(code)
-      @src << code
+      @pre << code
     end
 
     ##
