@@ -1,6 +1,7 @@
 $TESTING = true
 
-$0 = __FILE__ if $0 =~ /-e|\(eval\)/
+$0 = __FILE__ if $0 =~ /-e|\(eval\)|^$/
+$0 = $0.sub(Dir.pwd, '.')
 
 require 'inline'
 require 'tempfile'
@@ -13,15 +14,9 @@ File.umask(0)
 
 require 'pathname'
 
-$expand_paths     = Pathname.new(__FILE__).absolute?
 $inline_path      = './lib/inline.rb'
 $test_inline_path = './test/test_inline.rb'
 
-if $expand_paths then
-  $inline_path      = File.expand_path $inline_path
-  $test_inline_path = File.expand_path $test_inline_path
-end
-
 class InlineTestCase < MiniTest::Unit::TestCase
   def setup
     super
@@ -36,10 +31,6 @@ class InlineTestCase < MiniTest::Unit::TestCase
       ENV.delete 'INLINEDIR'
     end
   end
-
-  def test_stupid
-    #shuts test unit up
-  end
 end
 
 class TestDir < InlineTestCase
@@ -521,7 +512,7 @@ static VALUE method_name_equals(VALUE self, VALUE _value) {
   def util_generate(src, expected, expand_types=true)
     result = @builder.generate src, expand_types
     result = util_strip_lines result
-    result.gsub!(/\# line \d+/, '# line N')
+
     expected = "# line N \"#{$0}\"\n" + expected
     assert_equal(expected, result)
   end
@@ -533,7 +524,7 @@ static VALUE method_name_equals(VALUE self, VALUE _value) {
   def util_strip_lines(src)
     case src
     when String then
-      src.gsub(/\# line \d+/, '# line N')
+      src.gsub(/\# line \d+/, '# line N').gsub(Dir.pwd, '.')
     when Array then
       src.map do |chunk|
         util_strip_lines chunk
@@ -757,11 +748,15 @@ puts(s); return rb_str_new2(s)}"
 
     @builder.c "VALUE my_method() { return Qnil; }"
 
+    @builder.prefix "static int x;"
+
     windoze = "\n  __declspec(dllexport)" if Inline::WINDOZE
 
     expected = <<-EXT
 #include "ruby.h"
 
+static int x;
+
 # line N "#{$test_inline_path}"
 static VALUE allocate(VALUE self) {
  return (Qnil); }
@@ -774,7 +769,7 @@ static VALUE my_method(VALUE self) {
 #ifdef __cplusplus
 extern \"C\" {
 #endif#{windoze}
-  void Init_Inline_TestInline__TestC_eba5e5454322e22fe2310198ef14e43f() {
+  void Init_Inline_TestInline__TestC_3ab8c09639e499394bb1f0a0194a839f() {
     VALUE c = rb_cObject;
     c = rb_const_get(c, rb_intern("TestInline"));
     c = rb_const_get(c, rb_intern("TestC"));
@@ -807,21 +802,22 @@ extern \"C\" {
 
     expected = "# line N \"#{$0}\"\nstatic VALUE add(VALUE self, VALUE _a, VALUE _b) {\n  int a = FIX2INT(_a);\n  int b = FIX2INT(_b);\n return INT2FIX(a + b); }"
 
-    result.gsub!(/\# line \d+/, '# line N')
+    result = util_strip_lines result
 
     assert_equal expected, result
-    assert_equal [expected], @builder.src
+    assert_equal [expected], util_strip_lines(@builder.src)
   end
 
   def test_c_raw
     src = "static VALUE answer_raw(int argc, VALUE *argv, VALUE self) { return INT2NUM(42); }"
-    result = @builder.c_raw src.dup
 
-    result.gsub!(/\# line \d+/, '# line N')
+    result = @builder.c_raw src.dup
+    result = util_strip_lines result
+
     expected = "# line N \"#{$0}\"\n" + src
 
     assert_equal expected, result
-    assert_equal [expected], @builder.src
+    assert_equal [expected], util_strip_lines(@builder.src)
   end
 
   def test_map_c_const
@@ -874,6 +870,8 @@ extern \"C\" {
   end
 
   def test_build_good
+    skip "https://github.com/MagLev/maglev/issues/231" if maglev?
+
     code = util_simple_code(:DumbTest1, "long dumbpi() { return 314; }")
     util_test_build(code) do
       result = DumbTest1.new.dumbpi
@@ -983,6 +981,8 @@ EOR
 
 class TestModule < InlineTestCase
   def test_nested
+    skip "https://github.com/MagLev/maglev/issues/231" if maglev?
+
     Object.class_eval $test_module_code
     fb = Foo::Bar.new
     assert_equal(42, fb.forty_two_instance)
@@ -1003,12 +1003,16 @@ class TestModule < InlineTestCase
   end
 
   def test_argument_check_good
+    skip "https://github.com/MagLev/maglev/issues/231" if maglev?
+
     util_arity_check
     fb = Foo::Bar.new
     assert_equal 13, fb.arity6(1, 2, 3, 4, 5, "blah")
   end
 
   def test_argument_check_fewer
+    skip "https://github.com/MagLev/maglev/issues/231" if maglev?
+
     util_arity_check
     fb = Foo::Bar.new
 
@@ -1018,6 +1022,8 @@ class TestModule < InlineTestCase
   end
 
   def test_argument_check_more
+    skip "https://github.com/MagLev/maglev/issues/231" if maglev?
+
     util_arity_check
     fb = Foo::Bar.new
     assert_raises ArgumentError do
@@ -1026,6 +1032,8 @@ class TestModule < InlineTestCase
   end
 
   def test_inline
+    skip "https://github.com/MagLev/maglev/issues/231" if maglev?
+
     self.class.inline(:C) do |builder|
       builder.c "int add(int a, int b) { return a + b; }"
     end
